@@ -7,21 +7,24 @@ use crate::money::Currency;
 use crate::payment::PaymentProcessor;
 use crate::shipping::FulfillmentTypeSelection;
 
-pub async fn create_handler<T: Context + Send>(
-    ctx: &mut T,
+pub async fn create_handler<C: Context + Send>(
+    ctx: &mut C,
     currency: Currency,
     contact: Option<Contact>,
     shipping_address: Option<Address>,
-) -> Result<Checkout, Error> {
-    Checkout::new(ctx, currency, contact, shipping_address).await
+) -> Result<Checkout<C>, Error> {
+    let mut co = Checkout::new(ctx, currency, contact, shipping_address).await?;
+    co.populate_associated_fields(ctx).await?;
+
+    Ok(co)
 }
 
-pub async fn add_item_handler<T: Context + Send>(
-    ctx: &mut T,
+pub async fn add_item_handler<C: Context + Send>(
+    ctx: &mut C,
     checkout_id: &str,
     sku: String,
     quantity: u64,
-) -> Result<Checkout, Error> {
+) -> Result<Checkout<C>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -32,16 +35,17 @@ pub async fn add_item_handler<T: Context + Send>(
     let mut co = maybe_co.unwrap();
 
     co.add_item(ctx, sku, quantity).await?;
+    co.populate_associated_fields(ctx).await?;
 
     Ok(co)
 }
 
-pub async fn remove_item_handler<T: Context + Send>(
-    ctx: &mut T,
+pub async fn remove_item_handler<C: Context + Send>(
+    ctx: &mut C,
     checkout_id: &str,
     sku: String,
     quantity: u64,
-) -> Result<Checkout, Error> {
+) -> Result<Checkout<C>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -52,14 +56,15 @@ pub async fn remove_item_handler<T: Context + Send>(
     let mut co = maybe_co.unwrap();
 
     co.remove_item(ctx, sku, quantity).await?;
+    co.populate_associated_fields(ctx).await?;
 
     Ok(co)
 }
 
-pub async fn confirm_items_handler<T: Context + Send>(
-    ctx: &mut T,
+pub async fn confirm_items_handler<C: Context + Send>(
+    ctx: &mut C,
     checkout_id: &str,
-) -> Result<Checkout, Error> {
+) -> Result<Checkout<C>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -70,15 +75,16 @@ pub async fn confirm_items_handler<T: Context + Send>(
     let mut co = maybe_co.unwrap();
 
     co.confirm_items(ctx).await?;
+    co.populate_associated_fields(ctx).await?;
 
     Ok(co)
 }
 
-pub async fn update_contact_handler<T: Context + Send>(
-    ctx: &mut T,
+pub async fn update_contact_handler<C: Context + Send>(
+    ctx: &mut C,
     checkout_id: &str,
     contact: Contact,
-) -> Result<Checkout, Error> {
+) -> Result<Checkout<C>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -89,15 +95,16 @@ pub async fn update_contact_handler<T: Context + Send>(
     let mut co = maybe_co.unwrap();
 
     co.update_contact(ctx, contact).await?;
+    co.populate_associated_fields(ctx).await?;
 
     Ok(co)
 }
 
-pub async fn update_shipping_address_handler<T: Context + Send>(
-    ctx: &mut T,
+pub async fn update_shipping_address_handler<C: Context + Send>(
+    ctx: &mut C,
     checkout_id: &str,
     shipping_address: Address,
-) -> Result<Checkout, Error> {
+) -> Result<Checkout<C>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -108,15 +115,16 @@ pub async fn update_shipping_address_handler<T: Context + Send>(
     let mut co = maybe_co.unwrap();
 
     co.update_shipping_address(ctx, shipping_address).await?;
+    co.populate_associated_fields(ctx).await?;
 
     Ok(co)
 }
 
-pub async fn update_fulfillment_type_handler<T: Context + Send>(
-    ctx: &mut T,
+pub async fn update_fulfillment_type_handler<C: Context + Send>(
+    ctx: &mut C,
     checkout_id: &str,
     fulfillment_type: FulfillmentTypeSelection,
-) -> Result<Checkout, Error> {
+) -> Result<Checkout<C>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -127,15 +135,16 @@ pub async fn update_fulfillment_type_handler<T: Context + Send>(
     let mut co = maybe_co.unwrap();
 
     co.update_fulfillment_type(ctx, fulfillment_type).await?;
+    co.populate_associated_fields(ctx).await?;
 
     Ok(co)
 }
 
-pub async fn initiate_payment_handler<T: Context + Send>(
-    ctx: &mut T,
+pub async fn initiate_payment_handler<C: Context + Send>(
+    ctx: &mut C,
     checkout_id: &str,
-    args: <T as PaymentProcessor>::ProcessArgs,
-) -> Result<Checkout, Error> {
+    args: <C as PaymentProcessor>::ProcessArgs,
+) -> Result<Checkout<C>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -146,16 +155,17 @@ pub async fn initiate_payment_handler<T: Context + Send>(
     let mut co = maybe_co.unwrap();
 
     co.initiate_payment(ctx, args).await?;
+    co.populate_associated_fields(ctx).await?;
 
     Ok(co)
 }
 
-pub async fn initiate_order_handler<T: Context + Send>(
+pub async fn initiate_order_handler<C: Context + Send>(
     internal_ctx: &InternalContext,
-    ctx: &mut T,
+    ctx: &mut C,
     checkout_id: &str,
-    args: <T as PaymentProcessor>::ProcessArgs,
-) -> Result<Checkout, Error> {
+    args: <C as PaymentProcessor>::ProcessArgs,
+) -> Result<Checkout<C>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -166,6 +176,7 @@ pub async fn initiate_order_handler<T: Context + Send>(
     let mut co = maybe_co.unwrap();
 
     co.initiate_order(internal_ctx, ctx, args).await?;
+    co.populate_associated_fields(ctx).await?;
 
     Ok(co)
 }
