@@ -13,7 +13,7 @@ use crate::invoice::Invoice;
 use crate::item::Item;
 use crate::money::{Currency, Money};
 use crate::order::Order;
-use crate::payment::handlers as payment_handlers;
+use crate::payment::{handlers as payment_handlers, Payment};
 use crate::payment::{Initiator, PaymentProcessor};
 use crate::shipping::{FulfillmentType, FulfillmentTypeSelection, ShippingQuote};
 
@@ -28,7 +28,7 @@ pub enum State {
 }
 
 #[derive(Serialize, Deserialize, JsonSchema)]
-pub struct Checkout {
+pub struct Checkout<C: Context + Send> {
     pub id: String,
     pub state: State,
     pub currency: Currency,
@@ -41,19 +41,34 @@ pub struct Checkout {
     pub invoice: Option<Invoice>,
     pub payment_id: Option<String>,
     pub order_id: Option<String>,
+    // associations
+    pub payment: Option<
+        Payment<
+            <C as PaymentProcessor>::PrivateData,
+            <C as PaymentProcessor>::PublicData,
+            <C as PaymentProcessor>::ProcessArgs,
+        >,
+    >,
+    pub order: Option<Order<C>>,
 }
 
 #[async_trait]
 pub trait CheckoutStore {
-    async fn create_checkout(&mut self, co: &Checkout) -> Result<(), Error>;
-    async fn update_checkout(&mut self, co: &Checkout) -> Result<(), Error>;
-    async fn get_checkout(&mut self, id: &str) -> Result<Option<Checkout>, Error>;
-    async fn get_checkout_for_update(&mut self, id: &str) -> Result<Option<Checkout>, Error>;
+    async fn create_checkout<C: Context + Send>(&mut self, co: &Checkout<C>) -> Result<(), Error>;
+    async fn update_checkout<C: Context + Send>(&mut self, co: &Checkout<C>) -> Result<(), Error>;
+    async fn get_checkout<C: Context + Send>(
+        &mut self,
+        id: &str,
+    ) -> Result<Option<Checkout<C>>, Error>;
+    async fn get_checkout_for_update<C: Context + Send>(
+        &mut self,
+        id: &str,
+    ) -> Result<Option<Checkout<C>>, Error>;
 }
 
 // constructor
-impl Checkout {
-    pub async fn new<C: Context + Send>(
+impl<C: Context + Send> Checkout<C> {
+    pub async fn new(
         ctx: &mut C,
         currency: Currency,
         contact: Option<Contact>,
@@ -72,6 +87,9 @@ impl Checkout {
             invoice: None,
             payment_id: None,
             order_id: None,
+            // associations
+            payment: None,
+            order: None,
         };
 
         ctx.create_checkout(&co).await?;
@@ -81,17 +99,14 @@ impl Checkout {
 }
 
 // state transitions
-impl Checkout {
-    async fn enter_shopping_state<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
+impl<C: Context + Send> Checkout<C> {
+    async fn enter_shopping_state(&mut self, ctx: &mut C) -> Result<(), Error> {
         self.reset_shipping_options();
         ctx.free_items(&self.items).await?;
         Ok(self.state = State::Shopping)
     }
 
-    async fn enter_items_confirmed_state<C: Context + Send>(
-        &mut self,
-        ctx: &mut C,
-    ) -> Result<(), Error> {
+    async fn enter_items_confirmed_state(&mut self, ctx: &mut C) -> Result<(), Error> {
         self.reset_shipping_options();
         match ctx.reserve_items(&self.items).await? {
             ReserveResult::Success(deadline) => {
@@ -117,7 +132,7 @@ impl Checkout {
         }
     }
 
-    async fn enter_payment_in_progress_state<C: Context + Send>(
+    async fn enter_payment_in_progress_state(
         &mut self,
         ctx: &mut C,
         args: <C as PaymentProcessor>::ProcessArgs,
@@ -139,8 +154,8 @@ impl Checkout {
 }
 
 // private methods
-impl Checkout {
-    async fn ensure_shopping_state<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
+impl<C: Context + Send> Checkout<C> {
+    async fn ensure_shopping_state(&mut self, ctx: &mut C) -> Result<(), Error> {
         match self.state {
             State::Shopping => Ok(()),
             State::ItemsConfirmed(_) => self.enter_shopping_state(ctx).await,
@@ -150,10 +165,7 @@ impl Checkout {
         }
     }
 
-    async fn ensure_items_confirmed_state<C: Context + Send>(
-        &mut self,
-        ctx: &mut C,
-    ) -> Result<(), Error> {
+    async fn ensure_items_confirmed_state(&mut self, ctx: &mut C) -> Result<(), Error> {
         match self.state {
             State::ItemsConfirmed(_) => Ok(()),
             State::Shopping => self.enter_items_confirmed_state(ctx).await,
@@ -163,7 +175,7 @@ impl Checkout {
         }
     }
 
-    async fn ensure_payment_in_progress_state<C: Context + Send>(
+    async fn ensure_payment_in_progress_state(
         &mut self,
         ctx: &mut C,
         args: <C as PaymentProcessor>::ProcessArgs,
@@ -225,11 +237,11 @@ impl Checkout {
         }
     }
 
-    async fn update_invoice<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
+    async fn update_invoice(&mut self, ctx: &mut C) -> Result<(), Error> {
         Ok(self.invoice = Some(ctx.generate_invoice(self).await?))
     }
 
-    async fn update_item_prices<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
+    async fn update_item_prices(&mut self, ctx: &mut C) -> Result<(), Error> {
         let item_prices = ctx
             .calculate_item_prices(&self.currency, &self.promo_codes, &self.items)
             .await?;
@@ -254,13 +266,8 @@ impl Checkout {
 }
 
 // public methods
-impl Checkout {
-    pub async fn add_item<C: Context + Send>(
-        &mut self,
-        ctx: &mut C,
-        sku: String,
-        quantity: u64,
-    ) -> Result<(), Error> {
+impl<C: Context + Send> Checkout<C> {
+    pub async fn add_item(&mut self, ctx: &mut C, sku: String, quantity: u64) -> Result<(), Error> {
         self.ensure_shopping_state(ctx).await?;
 
         let mut is_added = false;
@@ -290,7 +297,7 @@ impl Checkout {
         ctx.update_checkout(self).await
     }
 
-    pub async fn remove_item<C: Context + Send>(
+    pub async fn remove_item(
         &mut self,
         ctx: &mut C,
         sku: String,
@@ -321,23 +328,19 @@ impl Checkout {
         ctx.update_checkout(self).await
     }
 
-    pub async fn confirm_items<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
+    pub async fn confirm_items(&mut self, ctx: &mut C) -> Result<(), Error> {
         self.ensure_items_confirmed_state(ctx).await?;
 
         ctx.update_checkout(self).await
     }
 
-    pub async fn update_contact<C: Context + Send>(
-        &mut self,
-        ctx: &mut C,
-        contact: Contact,
-    ) -> Result<(), Error> {
+    pub async fn update_contact(&mut self, ctx: &mut C, contact: Contact) -> Result<(), Error> {
         self.contact = Some(contact);
 
         ctx.update_checkout(self).await
     }
 
-    pub async fn update_shipping_address<C: Context + Send>(
+    pub async fn update_shipping_address(
         &mut self,
         ctx: &mut C,
         address: Address,
@@ -359,7 +362,7 @@ impl Checkout {
         ctx.update_checkout(self).await
     }
 
-    pub async fn update_fulfillment_type<C: Context + Send>(
+    pub async fn update_fulfillment_type(
         &mut self,
         ctx: &mut C,
         fulfillment_type: FulfillmentTypeSelection,
@@ -393,7 +396,7 @@ impl Checkout {
         ctx.update_checkout(self).await
     }
 
-    pub async fn initiate_payment<C: Context + Send>(
+    pub async fn initiate_payment(
         &mut self,
         ctx: &mut C,
         args: <C as PaymentProcessor>::ProcessArgs,
@@ -403,7 +406,7 @@ impl Checkout {
         ctx.update_checkout(self).await
     }
 
-    pub async fn initiate_order<C: Context + Send>(
+    pub async fn initiate_order(
         &mut self,
         internal_ctx: &InternalContext,
         ctx: &mut C,
@@ -423,7 +426,7 @@ impl Checkout {
         ctx.update_checkout(self).await
     }
 
-    pub async fn revert<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
+    pub async fn revert(&mut self, ctx: &mut C) -> Result<(), Error> {
         match self.state {
             State::Shopping => Ok(()),
             State::ItemsConfirmed(_) | State::PaymentInProgress(_) => {
@@ -443,7 +446,7 @@ impl Checkout {
         }
     }
 
-    pub async fn complete<C: Context + Send>(
+    pub async fn complete(
         &mut self,
         internal_ctx: &InternalContext,
         ctx: &mut C,
@@ -460,4 +463,22 @@ impl Checkout {
 
         ctx.update_checkout(self).await
     }
+
+    // utility method to be called by handlers
+    pub async fn populate_associated_fields(&mut self, ctx: &mut C) -> Result<(), Error> {
+        if let Some(ref payment_id) = self.payment_id {
+            self.payment = ctx.get_payment(payment_id).await?;
+        }
+
+        if let Some(ref order_id) = self.order_id {
+            let mut order = ctx.get_order(order_id).await?;
+            if let Some(ref mut od) = order {
+                od.populate_associated_fields(ctx).await?;
+            }
+
+            self.order = order;
+        }
+
+        Ok(())
+    }
 }
