@@ -13,15 +13,15 @@ use crate::{
     internal_context::InternalContext,
     invoice::Invoice,
     item::Item,
-    payment::PaymentProcessor,
     payment::{handlers as payment_handlers, Initiator},
+    payment::{Payment, PaymentProcessor},
     shipping::FulfillmentType,
 };
 
 pub mod handlers;
 
 #[derive(Serialize, Deserialize, JsonSchema)]
-pub struct Order {
+pub struct Order<C: Context + Send> {
     pub id: String,
     pub state: OrderState,
     pub state_reconciled: bool,
@@ -31,6 +31,14 @@ pub struct Order {
     pub fulfillment_type: FulfillmentType,
     pub invoice: Invoice,
     pub payment_id: Option<String>,
+    // associations
+    pub payment: Option<
+        Payment<
+            <C as PaymentProcessor>::PrivateData,
+            <C as PaymentProcessor>::PublicData,
+            <C as PaymentProcessor>::ProcessArgs,
+        >,
+    >,
 }
 
 #[derive(Serialize, Deserialize, JsonSchema)]
@@ -62,42 +70,51 @@ impl cloud_pubsub::FromPubSubMessage for LookupKey {
 
 #[async_trait]
 pub trait OrderStore {
-    async fn create_order(&mut self, order: &Order) -> Result<(), Error>;
-    async fn update_order(&mut self, order: &Order) -> Result<(), Error>;
-    async fn get_order_for_update(&mut self, id: &str) -> Result<Option<Order>, Error>;
-    async fn get_order(&mut self, id: &str) -> Result<Option<Order>, Error>;
+    async fn create_order<C: Context + Send>(&mut self, order: &Order<C>) -> Result<(), Error>;
+    async fn update_order<C: Context + Send>(&mut self, order: &Order<C>) -> Result<(), Error>;
+    async fn get_order_for_update<C: Context + Send>(
+        &mut self,
+        id: &str,
+    ) -> Result<Option<Order<C>>, Error>;
+    async fn get_order<C: Context + Send>(&mut self, id: &str) -> Result<Option<Order<C>>, Error>;
 }
 
 #[async_trait]
 pub trait OrderEvents {
-    async fn on_payment_in_progress(&mut self, _order: &Order) -> Result<(), Error> {
+    async fn on_payment_in_progress<C: Context + Send>(
+        &mut self,
+        _order: &Order<C>,
+    ) -> Result<(), Error> {
         Ok(())
     }
 
-    async fn on_pending_confirmation(&mut self, _order: &Order) -> Result<(), Error> {
+    async fn on_pending_confirmation<C: Context + Send>(
+        &mut self,
+        _order: &Order<C>,
+    ) -> Result<(), Error> {
         Ok(())
     }
 
-    async fn on_confirmed(&mut self, _order: &Order) -> Result<(), Error> {
+    async fn on_confirmed<C: Context + Send>(&mut self, _order: &Order<C>) -> Result<(), Error> {
         Ok(())
     }
 
-    async fn on_fulfilled(&mut self, _order: &Order) -> Result<(), Error> {
+    async fn on_fulfilled<C: Context + Send>(&mut self, _order: &Order<C>) -> Result<(), Error> {
         Ok(())
     }
 
-    async fn on_cancelled(&mut self, _order: &Order) -> Result<(), Error> {
+    async fn on_cancelled<C: Context + Send>(&mut self, _order: &Order<C>) -> Result<(), Error> {
         Ok(())
     }
 }
 
-impl Order {
-    pub async fn new_from_checkout<C: Context + Send>(
+impl<C: Context + Send> Order<C> {
+    pub async fn new_from_checkout(
         internal_ctx: &InternalContext,
         ctx: &mut C,
-        checkout: &Checkout,
+        checkout: &Checkout<C>,
     ) -> Result<Self, Error> {
-        let order = Order {
+        let order = Self {
             id: Uuid::new_v4().to_string(),
             state_reconciled: false,
             state: OrderState::PaymentInProgress,
@@ -107,6 +124,7 @@ impl Order {
             fulfillment_type: checkout.fulfillment_type.as_ref().unwrap().clone(),
             invoice: checkout.invoice.as_ref().unwrap().clone(),
             payment_id: checkout.payment_id.clone(),
+            payment: None,
         };
 
         ctx.create_order(&order).await?;
@@ -117,10 +135,10 @@ impl Order {
         Ok(order)
     }
 
-    pub async fn new_from_checkout_missing_payment<C: Context + Send>(
+    pub async fn new_from_checkout_missing_payment(
         internal_ctx: &InternalContext,
         ctx: &mut C,
-        checkout: &Checkout,
+        checkout: &Checkout<C>,
         payment_args: <C as PaymentProcessor>::ProcessArgs,
     ) -> Result<Self, Error> {
         let mut order = Order {
@@ -133,6 +151,7 @@ impl Order {
             fulfillment_type: checkout.fulfillment_type.as_ref().unwrap().clone(),
             invoice: checkout.invoice.as_ref().unwrap().clone(),
             payment_id: None,
+            payment: None,
         };
 
         ctx.create_order(&order).await?;
@@ -157,7 +176,7 @@ impl Order {
     }
 }
 
-impl Order {
+impl<C: Context + Send> Order<C> {
     async fn assert_reconciled(&self) -> Result<(), Error> {
         match self.state_reconciled {
             true => Ok(()),
@@ -246,8 +265,8 @@ impl Order {
     }
 }
 
-impl Order {
-    pub async fn prompt_confirmation<C: Context + Send>(
+impl<C: Context + Send> Order<C> {
+    pub async fn prompt_confirmation(
         &mut self,
         internal_ctx: &InternalContext,
         ctx: &mut C,
@@ -256,7 +275,7 @@ impl Order {
         ctx.update_order(self).await
     }
 
-    pub async fn confirm<C: Context + Send>(
+    pub async fn confirm(
         &mut self,
         internal_ctx: &InternalContext,
         ctx: &mut C,
@@ -265,7 +284,7 @@ impl Order {
         ctx.update_order(self).await
     }
 
-    pub async fn fulfill<C: Context + Send>(
+    pub async fn fulfill(
         &mut self,
         internal_ctx: &InternalContext,
         ctx: &mut C,
@@ -274,7 +293,7 @@ impl Order {
         ctx.update_order(self).await
     }
 
-    pub async fn cancel<C: Context + Send>(
+    pub async fn cancel(
         &mut self,
         internal_ctx: &InternalContext,
         ctx: &mut C,
@@ -283,7 +302,7 @@ impl Order {
         ctx.update_order(self).await
     }
 
-    pub async fn reconcile<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
+    pub async fn reconcile(&mut self, ctx: &mut C) -> Result<(), Error> {
         if self.state_reconciled == true {
             return Ok(());
         }
@@ -299,6 +318,15 @@ impl Order {
         self.state_reconciled = true;
         ctx.update_order(self).await
     }
+
+    // utility method to be called by handlers
+    pub async fn populate_associated_fields(&mut self, ctx: &mut C) -> Result<(), Error> {
+        if let Some(ref payment_id) = self.payment_id {
+            self.payment = ctx.get_payment(payment_id).await?;
+        }
+
+        Ok(())
+    }
 }
 
 async fn publish_state_change_event(
