@@ -9,7 +9,7 @@ use super::{
 
 pub async fn create_handler<'a, C: Context + Send>(
     ctx: &mut C,
-    initiator: Initiator<'a>,
+    initiator: Initiator<'a, C>,
     charge_amount: Money,
     args: <C as PaymentProcessor>::ProcessArgs,
 ) -> Result<
@@ -81,7 +81,9 @@ pub async fn process_handler<C: Context + Send>(
     let maybe_payment = ctx.get_payment_for_update(id).await?;
     if maybe_payment.is_none() {
         // this isn't supposed to happen.
-        return Err(new_not_found_error("the requested payment could not be found"));
+        return Err(new_not_found_error(
+            "the requested payment could not be found",
+        ));
     }
 
     let mut payment = maybe_payment.unwrap();
@@ -91,7 +93,9 @@ pub async fn process_handler<C: Context + Send>(
             let maybe_co = ctx.get_checkout_for_update(co_id).await?;
             if maybe_co.is_none() {
                 // this isn't supposed to happen.
-                return Err(new_not_found_error("the associated checkout could not be found"));
+                return Err(new_not_found_error(
+                    "the associated checkout could not be found",
+                ));
             }
 
             let mut checkout = maybe_co.unwrap();
@@ -107,7 +111,9 @@ pub async fn process_handler<C: Context + Send>(
             let maybe_od = ctx.get_order_for_update(od_id).await?;
             if maybe_od.is_none() {
                 // this isn't supposed to happen.
-                return Err(new_not_found_error("the associated order could not be found"));
+                return Err(new_not_found_error(
+                    "the associated order could not be found",
+                ));
             }
 
             let mut order = maybe_od.unwrap();
