@@ -1,5 +1,6 @@
 use async_trait::async_trait;
 use schemars::JsonSchema;
+use serde::de::DeserializeOwned;
 use serde::{Deserialize, Serialize};
 use uuid::Uuid;
 
@@ -35,16 +36,16 @@ pub enum InitiatorId {
     Order(String),
 }
 
-pub enum Initiator<'a> {
-    Checkout(&'a mut Checkout),
-    Order(&'a mut Order),
+pub enum Initiator<'a, C: Context + Send> {
+    Checkout(&'a mut Checkout<C>),
+    Order(&'a mut Order<C>),
 }
 
 #[async_trait]
 pub trait PaymentProcessor {
-    type ProcessArgs: Sync + Send;
-    type PrivateData: Sync + Send;
-    type PublicData: Sync + Send;
+    type ProcessArgs: Sync + Send + JsonSchema + Serialize + DeserializeOwned;
+    type PrivateData: Sync + Send + JsonSchema + Serialize + DeserializeOwned;
+    type PublicData: Sync + Send + JsonSchema + Serialize + DeserializeOwned;
 
     async fn initiate_payment(
         &mut self,
@@ -163,7 +164,7 @@ pub enum InternalPaymentState {
 // constructor
 async fn new<'a, C: Context + Send>(
     ctx: &mut C,
-    initiator: Initiator<'a>,
+    initiator: Initiator<'a, C>,
     charge_amount: Money,
     args: <C as PaymentProcessor>::ProcessArgs,
 ) -> Result<
@@ -244,7 +245,7 @@ pub async fn process<'a, C: Context + Send>(
         <C as PaymentProcessor>::PublicData,
         <C as PaymentProcessor>::ProcessArgs,
     >,
-    initiator: Initiator<'a>,
+    initiator: Initiator<'a, C>,
 ) -> Result<(), Error> {
     match &payment.state {
         PaymentState::Processing(args) => {
@@ -284,7 +285,7 @@ pub async fn cancel<'a, C: Context + Send>(
         <C as PaymentProcessor>::PublicData,
         <C as PaymentProcessor>::ProcessArgs,
     >,
-    initiator: Initiator<'a>,
+    initiator: Initiator<'a, C>,
 ) -> Result<(), Error> {
     match &payment.state {
         PaymentState::Cancelled => Ok(()),
